/*  
   Sequence Generator - seq-gen, version 1.3.2
   Copyright (c)1996-2004, Andrew Rambaut & Nick Grassly
   Department of Zoology, University of Oxford			
   All rights reserved.                          

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
   are met:

     1. Redistributions of source code must retain the above copyright
        notice, this list of conditions and the following disclaimer.

     2. Redistributions in binary form must reproduce the above copyright
        notice, this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.

     3. The names of its contributors may not be used to endorse or promote 
        products derived from this software without specific prior written 
        permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


   Any feedback is very welcome.
   http://evolve.zoo.ox.ac.uk/software/Seq-Gen/
   email: andrew.rambaut@zoo.ox.ac.uk
*/
 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <math.h>
#include "eigen.h"
#include "aamodels.h"

extern int SSF;
extern int C20;
extern int C60;
extern char site_aafreqFileName[256];   /* for PMSF model */
extern char aaFreqFileName[256];        /* for +F */ 
extern int user_defined_aafreq;

extern int whichAA(char c);
extern void readAAfreq(double *aaFreqPhyloBayes, int C60);
extern void rewriteAAfreq(double *aaFreqPhyloBayes); /* write aaFreq in the order in RAxML */
char *aminoAcids="ARNDCQEGHILKMFPSTWYV";

int aaFreqSet;
int aaModel = AA_NONE;
double aaFreq[NUM_AA];
double aaAddFreq[NUM_AA];
double aaMatrix[MAX_RATE_CATS][SQNUM_AA];
double aaVector[NUM_AA];
double aaRelativeRate[NUM_AA_REL_RATES];
static double Qij[SQNUM_AA], Cijk[CUNUM_AA], Root[NUM_AA];

void SetupAAMatrix();
void SetRelativeRates(double *inRelativeRates);
void SetFrequencies(double *inFrequencies);
void CheckAAFrequencies();


/* the aa in C20Frequenciesis in the same order as in RAxML */
static double C20Frequencies[20][NUM_AA] = {
  {0.0862413, 0.0171944, 0.0791293, 0.0329909, 0.0130505, 0.0169047, 0.0184527, 0.0366905, 0.0108013, 0.00979071, 0.0112826, 0.0220195, 0.00878215, 0.00441553, 0.0189273, 0.317815, 0.27117, 0.00153173, 0.00483429, 0.0179753},
  {0.203558, 0.00509808, 0.00770524, 0.00316561, 0.0348667, 0.00640441, 0.00708594, 0.0195236, 0.0024392, 0.115257, 0.0789777, 0.00423808, 0.0309187, 0.0112429, 0.0164189, 0.0496777, 0.111895, 0.00177626, 0.00484482, 0.284906},
  {0.0211547, 0.00149462, 0.0012755, 0.000549287, 0.00481886, 0.00123288, 0.00145396, 0.00114309, 0.00113464, 0.392846, 0.125064, 0.00135799, 0.0209789, 0.0128252, 0.00202472, 0.00262407, 0.0171914, 0.00115911, 0.0036028, 0.386068},
  {0.0376904, 0.28852, 0.0365411, 0.0109469, 0.00640738, 0.0893564, 0.0358365, 0.0191107, 0.0329514, 0.0101712, 0.0237496, 0.289763, 0.00965289, 0.00363498, 0.0105337, 0.0356314, 0.0355927, 0.00279252, 0.00665572, 0.0144622},
  {0.00845978, 0.00535899, 0.00725259, 0.00244879, 0.0084909, 0.00427815, 0.00250555, 0.00242771, 0.0433214, 0.0097713, 0.0380507, 0.0026741, 0.00807248, 0.342046, 0.00214187, 0.00804189, 0.00553221, 0.049484, 0.4375, 0.012141},
  {0.17599, 0.0290429, 0.0332846, 0.130126, 0.00175587, 0.0707184, 0.218217, 0.0409535, 0.0130708, 0.00856221, 0.0159531, 0.0542946, 0.00540458, 0.0025277, 0.037102, 0.0793481, 0.0540083, 0.00105921, 0.00362591, 0.0249553},
  {0.16344, 0.0195541, 0.0438702, 0.0374273, 0.00886599, 0.0137555, 0.0220612, 0.529672, 0.00900061, 0.00175694, 0.00611563, 0.0167118, 0.00293908, 0.00306413, 0.0126458, 0.0829343, 0.0142836, 0.00286407, 0.00323983, 0.00579857},
  {0.0917469, 0.0265853, 0.0290699, 0.0133819, 0.0284015, 0.0255085, 0.0196876, 0.0249899, 0.0449766, 0.0583556, 0.115501, 0.0164916, 0.0395995, 0.0998479, 0.0209916, 0.0736483, 0.0661518, 0.0246464, 0.0972327, 0.0831856},
  {0.0646701, 0.0988016, 0.0228907, 0.0168734, 0.00771176, 0.0996415, 0.0544978, 0.0148894, 0.0313852, 0.0505983, 0.184428, 0.0907931, 0.077484, 0.0219148, 0.0105004, 0.0321196, 0.0411766, 0.0084303, 0.0206106, 0.0505824},
  {0.0135994, 0.00434084, 0.00184694, 0.00079517, 0.010009, 0.00464208, 0.00180118, 0.00267946, 0.00724019, 0.0814027, 0.366142, 0.00251581, 0.0734965, 0.264097, 0.00389941, 0.00436688, 0.00752485, 0.0261565, 0.0660971, 0.0573473},
  {0.147804, 0.0842845, 0.072663, 0.0534743, 0.00488258, 0.0757166, 0.0727246, 0.0907726, 0.0262289, 0.00357811, 0.0126777, 0.105166, 0.00596218, 0.00299039, 0.0156558, 0.14599, 0.0634877, 0.00159285, 0.00507607, 0.00927198},
  {0.0186377, 0.00420552, 0.00198652, 0.00083297, 0.00549689, 0.00658901, 0.00202485, 0.00217135, 0.0023666, 0.202081, 0.437038, 0.00291207, 0.124186, 0.0385383, 0.00406723, 0.00461774, 0.0149904, 0.00268717, 0.00563241, 0.118938},
  {0.0477624, 0.0505743, 0.0209574, 0.0141349, 0.00757917, 0.0429297, 0.0462688, 0.00523279, 0.0165352, 0.17415, 0.112125, 0.0577575, 0.0330288, 0.0130691, 0.0124375, 0.0264989, 0.0951755, 0.00316605, 0.0112466, 0.20937},
  {0.416419, 0.00561008, 0.00917014, 0.00451317, 0.0406938, 0.00613205, 0.00632298, 0.0946185, 0.00310574, 0.00764432, 0.00998854, 0.00389418, 0.00693232, 0.00484384, 0.0187841, 0.236077, 0.0746275, 0.00121726, 0.00348258, 0.0459225},
  {0.0402296, 0.0735203, 0.103665, 0.0365524, 0.0124783, 0.0826558, 0.0372197, 0.0233618, 0.210831, 0.00934787, 0.0360561, 0.0482411, 0.010029, 0.0459095, 0.0098504, 0.0533383, 0.0310209, 0.0140077, 0.106438, 0.015248},
  {0.0323453, 0.0236283, 0.252045, 0.24315, 0.00359763, 0.0330831, 0.0710274, 0.101607, 0.0366225, 0.00314108, 0.00519805, 0.0470129, 0.00240287, 0.00244293, 0.00948378, 0.0848355, 0.0359083, 0.000873093, 0.00672477, 0.00487046},
  {0.147626, 0.0334507, 0.0211973, 0.0403052, 0.00323272, 0.0371554, 0.0576893, 0.0330851, 0.0146393, 0.0108267, 0.0256201, 0.0451351, 0.00586514, 0.00471772, 0.347371, 0.0892065, 0.0485899, 0.00163587, 0.00441772, 0.0282336},
  {0.103145, 0.0717748, 0.0435172, 0.0386402, 0.00617625, 0.0786603, 0.0923369, 0.0202338, 0.0246762, 0.0376904, 0.0376284, 0.0921699, 0.0161883, 0.00676664, 0.0128302, 0.095145, 0.137857, 0.00221447, 0.00830416, 0.0740454},
  {0.0837543, 0.0899383, 0.0518811, 0.0804871, 0.00207351, 0.14565, 0.194776, 0.022903, 0.0268459, 0.00740798, 0.019025, 0.145929, 0.00673952, 0.00230634, 0.0085616, 0.045574, 0.0451081, 0.00107713, 0.00493253, 0.0150303},
  {0.0578736, 0.0138314, 0.0491422, 0.294674, 0.00111308, 0.059825, 0.34021, 0.0293911, 0.0139817, 0.00305257, 0.00626119, 0.0363365, 0.0027296, 0.00170349, 0.0156106, 0.0358045, 0.0249942, 0.000866434, 0.0038313, 0.00876742}
};

/* C60 is in RAxML AA order */
const double C60Frequencies[60][20] = {
{0.153436000,0.044438900,0.079672700,0.054675700,0.004730660,0.051433300,0.052932400,0.110378000,0.017448000,0.005034390,0.013029400,0.060392900,0.007555060,0.003555430,0.024952400,0.202963000,0.095766800,0.001444450,0.005980030,0.010180900},
{0.028198500,0.303106000,0.031295500,0.009154930,0.001950350,0.093988400,0.038853000,0.008402830,0.015538500,0.010787300,0.021778700,0.347604000,0.010990500,0.001591930,0.007153990,0.019747900,0.032835200,0.000920999,0.002571400,0.013530300},
{0.008368070,0.000731977,0.000612345,0.000222837,0.002043390,0.000949869,0.000473154,0.000482575,0.000518999,0.376845000,0.260833000,0.000629617,0.031570100,0.012398400,0.000959592,0.000974638,0.004999080,0.000865776,0.001713230,0.293808000},
{0.222723000,0.006484610,0.006120650,0.000799759,0.164029000,0.005105190,0.002728080,0.020270300,0.003718390,0.045540600,0.088335000,0.002283290,0.034809500,0.022866700,0.003547160,0.085004000,0.101285000,0.004842480,0.009650000,0.169858000},
{0.041214000,0.006762710,0.005106770,0.001743440,0.020471600,0.005753850,0.003726340,0.006910750,0.018029400,0.115428000,0.169356000,0.004290030,0.041406700,0.223900000,0.005841640,0.014910600,0.023954800,0.033223700,0.137935000,0.120034000},
{0.048055000,0.030843800,0.094062900,0.208461000,0.003780180,0.074767700,0.185518000,0.019140200,0.087216200,0.009468540,0.027734100,0.037574100,0.008830840,0.019600100,0.008126780,0.043968100,0.032458900,0.003466570,0.038750000,0.018176900},
{0.006284870,0.002624690,0.003034250,0.000532415,0.007302760,0.003440910,0.000974149,0.001957820,0.010222500,0.018059200,0.117906000,0.001620590,0.023472200,0.397455000,0.002016560,0.005690330,0.003709180,0.059863900,0.318557000,0.015275400},
{0.181501000,0.002684540,0.014848500,0.002514550,0.420563000,0.001409700,0.000708814,0.046185400,0.001437460,0.004174550,0.009831050,0.000647425,0.004161140,0.006897640,0.003876720,0.186454000,0.068719000,0.002708350,0.006103300,0.034574200},
{0.060074100,0.036764300,0.013486900,0.017057200,0.007071980,0.014247000,0.012748700,0.034356400,0.030585900,0.020457100,0.099455100,0.021236700,0.031816600,0.114091000,0.029762800,0.050579300,0.033936800,0.231281000,0.119249000,0.021742200},
{0.070839500,0.047409800,0.041682200,0.032448300,0.013164100,0.049487500,0.050826400,0.018330900,0.056727300,0.065036900,0.128226000,0.034361800,0.039036300,0.059436000,0.013560800,0.055134300,0.064226000,0.013711800,0.067393400,0.078961000},
{0.061768900,0.007633290,0.030308200,0.343023000,0.000719984,0.030785600,0.379251000,0.028465900,0.007959210,0.001699960,0.003994530,0.021607700,0.001973430,0.000981419,0.017479100,0.033783200,0.020342700,0.000613027,0.001710280,0.005899230},
{0.042156000,0.104207000,0.028698100,0.016438500,0.004445030,0.139369000,0.053194900,0.013471100,0.017776500,0.026772800,0.196724000,0.132374000,0.118283000,0.008672830,0.005183790,0.025585300,0.033329200,0.004585230,0.007028150,0.021706700},
{0.281481000,0.010036700,0.017286800,0.006438570,0.025833800,0.013310200,0.011504600,0.027005500,0.005462970,0.018821600,0.019099300,0.009871280,0.015872000,0.005048170,0.012951000,0.188681000,0.242710000,0.001227460,0.003605290,0.083752400},
{0.276919000,0.001722700,0.002131530,0.001167250,0.031829300,0.001821630,0.002475250,0.019964700,0.000517086,0.098310900,0.048926400,0.001623220,0.017341500,0.007084390,0.007017970,0.033634900,0.081414100,0.000711814,0.003294230,0.362092000},
{0.157780000,0.111214000,0.057040300,0.064829000,0.005331810,0.106537000,0.091358700,0.090621000,0.053381000,0.002917160,0.015622600,0.078214900,0.004575900,0.002504780,0.006707780,0.092931000,0.039312300,0.002857580,0.007759030,0.008504090},
{0.059373500,0.035474100,0.115118000,0.218948000,0.001533220,0.068875200,0.181942000,0.081370700,0.022047800,0.002099360,0.005619130,0.075017200,0.002187170,0.001083830,0.010973700,0.072644900,0.038023800,0.000734646,0.002666490,0.004266970},
{0.097806600,0.026557600,0.010184400,0.012078100,0.006413840,0.030787600,0.029128300,0.012891300,0.012803700,0.072390400,0.127944000,0.024563100,0.030326700,0.019896400,0.272352000,0.035054900,0.048455700,0.004684250,0.010477400,0.115203000},
{0.012402300,0.003068040,0.000923910,0.000603732,0.004188570,0.003295740,0.001252400,0.001130680,0.001354210,0.234417000,0.455056000,0.001671820,0.066730800,0.061061500,0.003707620,0.001942090,0.006761290,0.003893720,0.007491180,0.129048000},
{0.079423100,0.129474000,0.066279300,0.058723600,0.001991950,0.114388000,0.124690000,0.032543200,0.023860500,0.003627720,0.009798800,0.214760000,0.004184620,0.001287000,0.014241000,0.061580700,0.047733400,0.000652537,0.002942020,0.007818720},
{0.024814900,0.008355290,0.188892000,0.427883000,0.002783970,0.021077800,0.143239000,0.064396800,0.018573700,0.002250690,0.003455860,0.017927400,0.001571450,0.001468040,0.007376800,0.037700300,0.018776800,0.000589186,0.004260270,0.004607270},
{0.001700340,0.006067430,0.000422290,0.001071150,0.002905940,0.001642420,0.001173170,0.003557960,0.002763050,0.001229120,0.012742100,0.000427380,0.002567130,0.051337700,0.001353670,0.001187170,0.001403310,0.864044000,0.039091300,0.003313730},
{0.046836100,0.063979700,0.020560400,0.018561600,0.005995410,0.055703100,0.070543600,0.004543530,0.015206300,0.155061000,0.082425300,0.086624800,0.024585400,0.008017720,0.008148560,0.023702600,0.096205400,0.001836870,0.006713170,0.204749000},
{0.025876500,0.020109700,0.029838400,0.010703700,0.014250400,0.015852900,0.010565000,0.007306500,0.141108000,0.011477800,0.040799200,0.011917900,0.009879900,0.187643000,0.005122880,0.027570000,0.017076500,0.040512500,0.353639000,0.018750200},
{0.029628500,0.004640030,0.003494440,0.000885102,0.009004650,0.005548110,0.003304650,0.002796950,0.005070150,0.258340000,0.266809000,0.004669090,0.077082500,0.040879800,0.002691820,0.006853810,0.032226600,0.003550610,0.015335300,0.227190000},
{0.055572600,0.009844790,0.040906400,0.014039000,0.009741860,0.006872770,0.006944320,0.015795700,0.004163130,0.006982650,0.007527120,0.013922500,0.005876270,0.003449670,0.011973300,0.348247000,0.421366000,0.001006150,0.002657680,0.013111900},
{0.068267100,0.061520700,0.053066100,0.036027900,0.014143300,0.061227400,0.049741500,0.026869700,0.112767000,0.013264700,0.054449400,0.048260900,0.017003400,0.080337600,0.019195000,0.067184000,0.044399600,0.019995800,0.125507000,0.026771400},
{0.079261900,0.063837700,0.063528900,0.043664600,0.004950330,0.066636500,0.082963900,0.018342900,0.023316900,0.024942700,0.022148300,0.093257800,0.012089300,0.004913110,0.012636000,0.133485000,0.191675000,0.001804010,0.006235310,0.050310200},
{0.073175900,0.210534000,0.032420100,0.011000700,0.012345900,0.085895200,0.034994300,0.022450900,0.038690300,0.024622600,0.050830700,0.178334000,0.018574100,0.009314880,0.014872300,0.060318100,0.064957500,0.005104640,0.013059700,0.038504000},
{0.087840300,0.011033200,0.006080120,0.003280390,0.017114700,0.010983200,0.010146600,0.008709090,0.005490220,0.198776000,0.175646000,0.008209690,0.041723300,0.019195400,0.011128400,0.020986300,0.069771900,0.003174400,0.008190550,0.282520000},
{0.099021600,0.034935200,0.021115000,0.011879800,0.010899600,0.055771100,0.027900000,0.024025000,0.012344500,0.077656500,0.235451000,0.032281800,0.120767000,0.021444200,0.007565550,0.052417000,0.064978500,0.004707580,0.007732870,0.077106700},
{0.060164100,0.016199500,0.278352000,0.033718900,0.031506700,0.021064600,0.005983950,0.054308100,0.053152400,0.007065080,0.007069810,0.013959800,0.008829870,0.006952590,0.007583430,0.282980000,0.086031700,0.001496660,0.013484900,0.010095400},
{0.004978170,0.001841230,0.000701221,0.000531537,0.005297870,0.002408990,0.000763055,0.001505130,0.004157520,0.044382900,0.441742000,0.001161510,0.060280700,0.335112000,0.002784770,0.002579580,0.003028850,0.017130300,0.045845600,0.023767700},
{0.025199700,0.111447000,0.014203200,0.004101230,0.009709950,0.062007100,0.026257200,0.003806730,0.043193900,0.097404300,0.244720000,0.082431300,0.053932300,0.042909200,0.005265850,0.009609310,0.025118300,0.014657200,0.045696500,0.078330300},
{0.023036200,0.001474870,0.001353440,0.000626444,0.004858010,0.000987005,0.001576260,0.001156530,0.000889924,0.395290000,0.057653700,0.001466350,0.014098700,0.007212700,0.002017790,0.002877020,0.020558100,0.000547769,0.001953910,0.460366000},
{0.140878000,0.029780800,0.017129800,0.028507700,0.003221370,0.032063200,0.042383900,0.029955800,0.013132100,0.006691450,0.019512000,0.038378200,0.003627690,0.004123110,0.438347000,0.085140000,0.042276600,0.001323690,0.003708760,0.019819500},
{0.444249000,0.005021660,0.010230500,0.005719300,0.023540500,0.005599760,0.006488990,0.082268800,0.002550570,0.003361510,0.004099010,0.003809710,0.002868310,0.002441320,0.016289100,0.299997000,0.055966500,0.000773543,0.002063980,0.022660800},
{0.089871800,0.007095830,0.013006800,0.012916700,0.004413150,0.002380650,0.005895700,0.808756000,0.001651790,0.000433928,0.001556450,0.003393900,0.000425342,0.000807357,0.003412810,0.036287700,0.003288750,0.001522390,0.000853745,0.002028960},
{0.055084000,0.047225400,0.187783000,0.127380000,0.003582490,0.052796900,0.065588500,0.063760800,0.040488300,0.007557420,0.013630500,0.086768300,0.008168420,0.004037500,0.011068200,0.126338000,0.075254400,0.001356370,0.011859000,0.010272800},
{0.011768100,0.044255900,0.084414500,0.014471200,0.007038830,0.103834000,0.011090100,0.004962660,0.433719000,0.006133700,0.029879500,0.013792900,0.007623760,0.033826600,0.008134610,0.014057100,0.010827700,0.008068310,0.143725000,0.008375780},
{0.015928600,0.004809870,0.003269260,0.001096690,0.008051990,0.013455200,0.002132420,0.002508640,0.004919210,0.050154400,0.530763000,0.003559940,0.216009000,0.074365100,0.004524740,0.006692220,0.011909200,0.007092810,0.010656500,0.028101200},
{0.019597300,0.010514300,0.328910000,0.309985000,0.003453900,0.011619700,0.025077800,0.062752900,0.029596100,0.003265040,0.002824690,0.024096400,0.000842506,0.001970650,0.004906280,0.106498000,0.043805400,0.000633396,0.005619800,0.004030200},
{0.083380400,0.012587100,0.096982400,0.068682100,0.008198110,0.012152100,0.022741500,0.098229200,0.007395490,0.001747120,0.003965310,0.012934200,0.001955800,0.002413260,0.035592400,0.311561000,0.211337000,0.001632900,0.001799110,0.004713860},
{0.018140900,0.412966000,0.023320500,0.003333350,0.008514360,0.052669400,0.009653190,0.022455300,0.037523900,0.003509050,0.014914700,0.320807000,0.004609890,0.003542690,0.008719750,0.026230900,0.013179100,0.003476700,0.007958820,0.004474650},
{0.249423000,0.018548200,0.016412000,0.016923400,0.012286300,0.022850200,0.037049100,0.034746800,0.008706960,0.059571800,0.045106500,0.017706500,0.020455600,0.007736090,0.068640400,0.088929600,0.098601700,0.002860390,0.006193850,0.167252000},
{0.141974000,0.037394600,0.057629700,0.053745200,0.006885670,0.028624000,0.040754000,0.398811000,0.015289600,0.001662760,0.009234830,0.031427400,0.005542550,0.004028610,0.018032900,0.112373000,0.024247800,0.002590910,0.004905420,0.004846290},
{0.017890300,0.195884000,0.015585400,0.003105430,0.029030400,0.105182000,0.004050340,0.010048000,0.125270000,0.001670800,0.072235700,0.023334000,0.011614200,0.023891300,0.000993842,0.018167600,0.018626000,0.226055000,0.085978700,0.011386500},
{0.145476000,0.042097900,0.040042000,0.129425000,0.001418630,0.090646900,0.247135000,0.031965100,0.013042600,0.005852540,0.012359300,0.081815400,0.004417890,0.001755210,0.015113600,0.065668800,0.051128900,0.000773144,0.002925840,0.016940100},
{0.016979900,0.024234700,0.131805000,0.104366000,0.002208720,0.026935000,0.037638000,0.540447000,0.018113700,0.000745968,0.002114700,0.050861800,0.000947377,0.000678059,0.003875440,0.029703000,0.004583620,0.000603189,0.001570410,0.001589170},
{0.040264600,0.115202000,0.032382900,0.029396800,0.003938870,0.249701000,0.160352000,0.012926000,0.061796800,0.009849130,0.035491900,0.144880000,0.012481900,0.004115340,0.004337420,0.024324700,0.030564500,0.002667660,0.009722780,0.015602700},
{0.225691000,0.052341700,0.024430900,0.063712500,0.004339010,0.057815900,0.115483000,0.086733500,0.013106700,0.008508620,0.019331400,0.066046900,0.006487720,0.002744010,0.061114900,0.107088000,0.050767700,0.001369590,0.002898290,0.029988300},
{0.003316420,0.001531080,0.003083020,0.000826647,0.005189070,0.001102490,0.000513413,0.001043280,0.027845100,0.004189530,0.011121200,0.000714992,0.002362180,0.380176000,0.000836508,0.003587670,0.002360890,0.033334700,0.510789000,0.006076630},
{0.199501000,0.023607900,0.039225500,0.009495510,0.058459000,0.025426500,0.012553500,0.093978700,0.034185700,0.014021000,0.044938800,0.011872300,0.024699100,0.063443400,0.014538500,0.166392000,0.053315900,0.012980300,0.060634600,0.036730300},
{0.031944900,0.101167000,0.208471000,0.037807500,0.006604030,0.076637300,0.027948800,0.036554100,0.208864000,0.004754230,0.015654600,0.086866500,0.004325330,0.010891600,0.006089960,0.057765700,0.030205100,0.002600190,0.038789700,0.006058520},
{0.077680000,0.014251900,0.040321700,0.008065170,0.014009300,0.017999600,0.011262200,0.013686800,0.013373000,0.123964000,0.072467100,0.012914500,0.042074500,0.017358500,0.011708400,0.092272400,0.231690000,0.002815360,0.014172700,0.167914000},
{0.118366000,0.080519300,0.025952500,0.049559500,0.003562480,0.120492000,0.153759000,0.019499300,0.022937300,0.030266100,0.057125100,0.098230400,0.017172700,0.006866570,0.017515300,0.048658800,0.063579600,0.002300830,0.008302740,0.055333600},
{0.052856000,0.019356900,0.026474400,0.209276000,0.000862588,0.121241000,0.402419000,0.015583800,0.012414900,0.005486480,0.009025650,0.049701700,0.004235710,0.001265070,0.006318560,0.019726300,0.023546400,0.000838161,0.003394870,0.015976400},
{0.034436600,0.042622200,0.163672000,0.113901000,0.002098600,0.060541400,0.054178000,0.336164000,0.046177700,0.000346342,0.004835570,0.066755300,0.001970450,0.003155760,0.004036980,0.048117300,0.008914810,0.000651010,0.005414560,0.002011060},
{0.115309000,0.015127900,0.045847700,0.175552000,0.001496240,0.036673100,0.174941000,0.039418100,0.013240200,0.005691300,0.010141000,0.043311800,0.003033210,0.001570020,0.166580000,0.087153600,0.046826100,0.000751570,0.003143270,0.014193200},
{0.386515000,0.003757930,0.003042050,0.002236680,0.021892800,0.002146470,0.003138780,0.369435000,0.001467290,0.008537610,0.012725700,0.001884050,0.008058170,0.003928140,0.015868800,0.080887700,0.030519600,0.000992288,0.001902030,0.041063500},
{0.014657100,0.002884130,0.001299830,0.000521057,0.002431790,0.004936280,0.001487440,0.002095330,0.001018190,0.191390000,0.443280000,0.002289840,0.221743000,0.009163750,0.000768515,0.002725150,0.017099700,0.000877938,0.001475600,0.077855700}
};


/* LG model for amino acid evolution */
double LGRelativeRates[NUM_AA_REL_RATES], LGFrequencies[NUM_AA];

void initilizeLG()
{
  int i, j, r;
  double aaMut[20][20], a[20][20];

	aaMut[1][0]=0.425093; 
	aaMut[2][0]=0.276818; aaMut[2][1]=0.751878; 
	aaMut[3][0]=0.395144; aaMut[3][1]=0.123954; aaMut[3][2]=5.076149; 
	aaMut[4][0]=2.489084; aaMut[4][1]=0.534551; aaMut[4][2]=0.528768; aaMut[4][3]=0.062556; 
	aaMut[5][0]=0.969894; aaMut[5][1]=2.807908; aaMut[5][2]=1.695752; aaMut[5][3]=0.523386; aaMut[5][4]=0.084808; 
	aaMut[6][0]=1.038545; aaMut[6][1]=0.363970; aaMut[6][2]=0.541712; aaMut[6][3]=5.243870; aaMut[6][4]=0.003499; aaMut[6][5]=4.128591; 
	aaMut[7][0]=2.066040; aaMut[7][1]=0.390192; aaMut[7][2]=1.437645; aaMut[7][3]=0.844926; aaMut[7][4]=0.569265; aaMut[7][5]=0.267959; aaMut[7][6]=0.348847; 
	aaMut[8][0]=0.358858; aaMut[8][1]=2.426601; aaMut[8][2]=4.509238; aaMut[8][3]=0.927114; aaMut[8][4]=0.640543; aaMut[8][5]=4.813505; aaMut[8][6]=0.423881; aaMut[8][7]=0.311484; 
	aaMut[9][0]=0.149830; aaMut[9][1]=0.126991; aaMut[9][2]=0.191503; aaMut[9][3]=0.010690; aaMut[9][4]=0.320627; aaMut[9][5]=0.072854; aaMut[9][6]=0.044265; aaMut[9][7]=0.008705; aaMut[9][8]=0.108882; 
	aaMut[10][0]=0.395337; aaMut[10][1]=0.301848; aaMut[10][2]=0.068427; aaMut[10][3]=0.015076; aaMut[10][4]=0.594007; aaMut[10][5]=0.582457; aaMut[10][6]=0.069673; aaMut[10][7]=0.044261; aaMut[10][8]=0.366317; aaMut[10][9]=4.145067; 
	aaMut[11][0]=0.536518; aaMut[11][1]=6.326067; aaMut[11][2]=2.145078; aaMut[11][3]=0.282959; aaMut[11][4]=0.013266; aaMut[11][5]=3.234294; aaMut[11][6]=1.807177; aaMut[11][7]=0.296636; aaMut[11][8]=0.697264; aaMut[11][9]=0.159069; aaMut[11][10]=0.137500; 
	aaMut[12][0]=1.124035; aaMut[12][1]=0.484133; aaMut[12][2]=0.371004; aaMut[12][3]=0.025548; aaMut[12][4]=0.893680; aaMut[12][5]=1.672569; aaMut[12][6]=0.173735; aaMut[12][7]=0.139538; aaMut[12][8]=0.442472; aaMut[12][9]=4.273607; aaMut[12][10]=6.312358; aaMut[12][11]=0.656604; 
	aaMut[13][0]=0.253701; aaMut[13][1]=0.052722; aaMut[13][2]=0.089525; aaMut[13][3]=0.017416; aaMut[13][4]=1.105251; aaMut[13][5]=0.035855; aaMut[13][6]=0.018811; aaMut[13][7]=0.089586; aaMut[13][8]=0.682139; aaMut[13][9]=1.112727; aaMut[13][10]=2.592692; aaMut[13][11]=0.023918; aaMut[13][12]=1.798853; 
	aaMut[14][0]=1.177651; aaMut[14][1]=0.332533; aaMut[14][2]=0.161787; aaMut[14][3]=0.394456; aaMut[14][4]=0.075382; aaMut[14][5]=0.624294; aaMut[14][6]=0.419409; aaMut[14][7]=0.196961; aaMut[14][8]=0.508851; aaMut[14][9]=0.078281; aaMut[14][10]=0.249060; aaMut[14][11]=0.390322; aaMut[14][12]=0.099849; aaMut[14][13]=0.094464; 
	aaMut[15][0]=4.727182; aaMut[15][1]=0.858151; aaMut[15][2]=4.008358; aaMut[15][3]=1.240275; aaMut[15][4]=2.784478; aaMut[15][5]=1.223828; aaMut[15][6]=0.611973; aaMut[15][7]=1.739990; aaMut[15][8]=0.990012; aaMut[15][9]=0.064105; aaMut[15][10]=0.182287; aaMut[15][11]=0.748683; aaMut[15][12]=0.346960; aaMut[15][13]=0.361819; aaMut[15][14]=1.338132; 
	aaMut[16][0]=2.139501; aaMut[16][1]=0.578987; aaMut[16][2]=2.000679; aaMut[16][3]=0.425860; aaMut[16][4]=1.143480; aaMut[16][5]=1.080136; aaMut[16][6]=0.604545; aaMut[16][7]=0.129836; aaMut[16][8]=0.584262; aaMut[16][9]=1.033739; aaMut[16][10]=0.302936; aaMut[16][11]=1.136863; aaMut[16][12]=2.020366; aaMut[16][13]=0.165001; aaMut[16][14]=0.571468; aaMut[16][15]=6.472279; 
	aaMut[17][0]=0.180717; aaMut[17][1]=0.593607; aaMut[17][2]=0.045376; aaMut[17][3]=0.029890; aaMut[17][4]=0.670128; aaMut[17][5]=0.236199; aaMut[17][6]=0.077852; aaMut[17][7]=0.268491; aaMut[17][8]=0.597054; aaMut[17][9]=0.111660; aaMut[17][10]=0.619632; aaMut[17][11]=0.049906; aaMut[17][12]=0.696175; aaMut[17][13]=2.457121; aaMut[17][14]=0.095131; aaMut[17][15]=0.248862; aaMut[17][16]=0.140825; 
	aaMut[18][0]=0.218959; aaMut[18][1]=0.314440; aaMut[18][2]=0.612025; aaMut[18][3]=0.135107; aaMut[18][4]=1.165532; aaMut[18][5]=0.257336; aaMut[18][6]=0.120037; aaMut[18][7]=0.054679; aaMut[18][8]=5.306834; aaMut[18][9]=0.232523; aaMut[18][10]=0.299648; aaMut[18][11]=0.131932; aaMut[18][12]=0.481306; aaMut[18][13]=7.803902; aaMut[18][14]=0.089613; aaMut[18][15]=0.400547; aaMut[18][16]=0.245841; aaMut[18][17]=3.151815; 
	aaMut[19][0]=2.547870; aaMut[19][1]=0.170887; aaMut[19][2]=0.083688; aaMut[19][3]=0.037967; aaMut[19][4]=1.959291; aaMut[19][5]=0.210332; aaMut[19][6]=0.245034; aaMut[19][7]=0.076701; aaMut[19][8]=0.119013; aaMut[19][9]=10.649107; aaMut[19][10]=1.702745; aaMut[19][11]=0.185202; aaMut[19][12]=1.898718; aaMut[19][13]=0.654683; aaMut[19][14]=0.296501; aaMut[19][15]=0.098369; aaMut[19][16]=2.188158; aaMut[19][17]=0.189510; aaMut[19][18]=0.249313; 
	
	/* from triangular exchange matrix to square exchange  matrix */
	for (i=0; i<20; i++) {
	  for (j=0; j<=i; j++) 
	    a[i][j] = aaMut[i][j];
	  
	  for (j=i+1; j<20; j++)
	    a[i][j] = aaMut[j][i];  
	}
	
	r = 0;
	for(i = 0; i < 19; i++)
	  {      
	    for(j = i + 1; j < 20; j++) 
	      LGRelativeRates[r++] = a[i][j];
	  }


	LGFrequencies[0]=0.079066; LGFrequencies[1]=0.055941; LGFrequencies[2]=0.041977; LGFrequencies[3]=0.053052; LGFrequencies[4]=0.012937; LGFrequencies[5]=0.040767; LGFrequencies[6]=0.071586; LGFrequencies[7]=0.057337; LGFrequencies[8]=0.022355; LGFrequencies[9]=0.062157; LGFrequencies[10]=0.099081; LGFrequencies[11]=0.064600; LGFrequencies[12]=0.022951; LGFrequencies[13]=0.042302; LGFrequencies[14]=0.044040; LGFrequencies[15]=0.061197; LGFrequencies[16]=0.053287; LGFrequencies[17]=0.012066; LGFrequencies[18]=0.034155; LGFrequencies[19]=0.069147; 
}



/* JTT model for amino acid evolution */
/* D.T. Jones, W.R. Taylor, and J.M. Thornton */
/* The rapid generation of mutation data matrices from protein sequences */
/* CABIOS  vol. 8 no. 3 1992 pp. 275-282 */
static double jttRelativeRates[NUM_AA_REL_RATES] = {
	0.531678, 0.557967, 0.827445, 0.574478, 0.556725, 1.066681, 1.740159, 0.219970, 0.361684, 0.310007, 0.369437, 0.469395, 0.138293, 1.959599, 3.887095, 4.582565, 0.084329, 0.139492, 2.924161,
	0.451095, 0.154899, 1.019843, 3.021995, 0.318483, 1.359652, 3.210671, 0.239195, 0.372261, 6.529255, 0.431045, 0.065314, 0.710489, 1.001551, 0.650282, 1.257961, 0.235601, 0.171995,
	5.549530, 0.313311, 0.768834, 0.578115, 0.773313, 4.025778, 0.491003, 0.137289, 2.529517, 0.330720, 0.073481, 0.121804, 5.057964, 2.351311, 0.027700, 0.700693, 0.164525,
	0.105625, 0.521646, 7.766557, 1.272434, 1.032342, 0.115968, 0.061486, 0.282466, 0.190001, 0.032522, 0.127164, 0.589268, 0.425159, 0.057466, 0.453952, 0.315261,
	0.091304, 0.053907, 0.546389, 0.724998, 0.150559, 0.164593, 0.049009, 0.409202, 0.678335, 0.123653, 2.155331, 0.469823, 1.104181, 2.114852, 0.621323,
	3.417706, 0.231294, 5.684080, 0.078270, 0.709004, 2.966732, 0.456901, 0.045683, 1.608126, 0.548807, 0.523825, 0.172206, 0.254745, 0.179771,
	1.115632, 0.243768, 0.111773, 0.097485, 1.731684, 0.175084, 0.043829, 0.191994, 0.312449, 0.331584, 0.114381, 0.063452, 0.465271,
	0.201696, 0.053769, 0.069492, 0.269840, 0.130379, 0.050212, 0.208081, 1.874296, 0.316862, 0.544180, 0.052500, 0.470140,
	0.181788, 0.540571, 0.525096, 0.329660, 0.453428, 1.141961, 0.743458, 0.477355, 0.128193, 5.848400, 0.121827,
	2.335139, 0.202562, 4.831666, 0.777090, 0.098580, 0.405119, 2.553806, 0.134510, 0.303445, 9.533943,
	0.146481, 3.856906, 2.500294, 1.060504, 0.592511, 0.272514, 0.530324, 0.241094, 1.761439,
	0.624581, 0.024521, 0.216345, 0.474478, 0.965641, 0.089134, 0.087904, 0.124066,
	0.436181, 0.164215, 0.285564, 2.114728, 0.201334, 0.189870, 3.038533,
	0.148483, 0.943971, 0.138904, 0.537922, 5.484236, 0.593478,
	2.788406, 1.176961, 0.069965, 0.113850, 0.211561,
	4.777647, 0.310927, 0.628608, 0.408532,
	0.080556, 0.201094, 1.14398,
	0.747889, 0.239697,
	0.165473
};

static double jttFrequencies[NUM_AA] = {
	0.076862, 0.051057, 0.042546, 0.051269, 0.020279, 0.041061, 0.061820, 0.074714, 0.022983, 0.052569, 0.091111, 0.059498, 0.023414, 0.040530, 0.050532, 0.068225, 0.058518, 0.014336, 0.032303, 0.066374
};
	
/* WAG model of amino acid evolution */
/* Whelan, S. and Goldman, N. (2001) A general empirical model of protein */
/* evolution derived from multiple protein families using a maximum-likelihood */
/* approach. Mol. Biol. Evol. 18, 691699. */
static double wagRelativeRates[NUM_AA_REL_RATES] = {
	0.610810, 0.569079, 0.821500, 1.141050, 1.011980, 1.756410, 1.572160, 0.354813, 0.219023, 0.443935, 1.005440, 0.989475, 0.233492, 1.594890, 3.733380, 2.349220, 0.125227, 0.268987, 2.221870, 
	0.711690, 0.165074, 0.585809, 3.360330, 0.488649, 0.650469, 2.362040, 0.206722, 0.551450, 5.925170, 0.758446, 0.116821, 0.753467, 1.357640, 0.613776, 1.294610, 0.423612, 0.280336, 
	6.013660, 0.296524, 1.716740, 1.056790, 1.253910, 4.378930, 0.615636, 0.147156, 3.334390, 0.224747, 0.110793, 0.217538, 4.394450, 2.257930, 0.078463, 1.208560, 0.221176, 
	0.033379, 0.691268, 6.833400, 0.961142, 1.032910, 0.043523, 0.093930, 0.533362, 0.116813, 0.052004, 0.472601, 1.192810, 0.417372, 0.146348, 0.363243, 0.169417, 
	0.109261, 0.023920, 0.341086, 0.275403, 0.189890, 0.428414, 0.083649, 0.437393, 0.441300, 0.122303, 1.560590, 0.570186, 0.795736, 0.604634, 1.114570, 
	6.048790, 0.366510, 4.749460, 0.131046, 0.964886, 4.308310, 1.705070, 0.110744, 1.036370, 1.141210, 0.954144, 0.243615, 0.252457, 0.333890, 
	0.630832, 0.635025, 0.141320, 0.172579, 2.867580, 0.353912, 0.092310, 0.755791, 0.782467, 0.914814, 0.172682, 0.217549, 0.655045, 
	0.276379, 0.034151, 0.068651, 0.415992, 0.194220, 0.055288, 0.273149, 1.486700, 0.251477, 0.374321, 0.114187, 0.209108, 
	0.152215, 0.555096, 0.992083, 0.450867, 0.756080, 0.771387, 0.822459, 0.525511, 0.289998, 4.290350, 0.131869, 
	3.517820, 0.360574, 4.714220, 1.177640, 0.111502, 0.353443, 1.615050, 0.234326, 0.468951, 8.659740, 
	0.287583, 5.375250, 2.348200, 0.462018, 0.382421, 0.364222, 0.740259, 0.443205, 1.997370, 
	1.032220, 0.098843, 0.619503, 1.073780, 1.537920, 0.152232, 0.147411, 0.342012, 
	1.320870, 0.194864, 0.556353, 1.681970, 0.570369, 0.473810, 2.282020, 
	0.179896, 0.606814, 0.191467, 1.699780, 7.154480, 0.725096, 
	1.786490, 0.885349, 0.156619, 0.239607, 0.351250, 
	4.847130, 0.578784, 0.872519, 0.258861, 
	0.126678, 0.325490, 1.547670, 
	2.763540, 0.409817, 
	0.347826
};

static double wagFrequencies[NUM_AA] = {
	0.0866, 0.0440, 0.0391, 0.0570, 0.0193, 0.0367, 0.0581, 0.0833, 0.0244, 0.0485, 0.0862, 0.0620, 0.0195, 0.0384, 0.0458, 0.0695, 0.0610, 0.0144, 0.0353, 0.0709
};

/* Dayhoff, M.O., Schwartz, R.M., Orcutt, B.C. (1978)*/
/* A model of evolutionary change in proteins.*/
/* Dayhoff, M.O. (ed.) Atlas of Protein Sequence Structur., Vol5, Suppl. 3,*/
/* National Biomedical Research Foundation, Washington DC, pp. 345-352.*/
static double dayhoffRelativeRates[NUM_AA_REL_RATES] = {
	0.267828, 0.984474, 1.199805, 0.360016, 0.887753, 1.961167, 2.386111, 0.228116, 0.653416, 0.406431, 0.258635, 0.717840, 0.183641, 2.485920, 4.051870, 3.680365, 0.000000, 0.244139, 2.059564,
	0.327059, 0.000000, 0.232374, 2.439939, 0.000000, 0.087791, 2.383148, 0.632629, 0.154924, 4.610124, 0.896321, 0.136906, 1.028313, 1.531590, 0.265745, 2.001375, 0.078012, 0.240368,
	8.931515, 0.000000, 1.028509, 1.493409, 1.385352, 5.290024, 0.768024, 0.341113, 3.148371, 0.000000, 0.138503, 0.419244, 4.885892, 2.271697, 0.224968, 0.946940, 0.158067,
	0.00000, 1.348551, 11.388659, 1.240981, 0.868241, 0.239248, 0.000000, 0.716913, 0.000000, 0.000000, 0.133940, 0.956097, 0.660930, 0.000000, 0.000000, 0.178316,
	0.000000, 0.000000, 0.107278, 0.282729, 0.438074, 0.000000, 0.000000, 0.000000, 0.000000, 0.187550, 1.598356, 0.162366, 0.000000, 0.953164, 0.484678,
	7.086022, 0.281581, 6.011613, 0.180393, 0.730772, 1.519078, 1.127499, 0.000000, 1.526188, 0.561828, 0.525651, 0.000000, 0.000000, 0.346983,
	0.811907, 0.439469, 0.609526, 0.112880, 0.830078, 0.304803, 0.000000, 0.507003, 0.793999, 0.340156, 0.000000, 0.214717, 0.36725,
	0.106802, 0.000000, 0.071514, 0.267683, 0.170372, 0.153478, 0.347153, 2.322243, 0.306662, 0.000000, 0.000000, 0.538165,
	0.076981, 0.443504, 0.270475, 0.000000, 0.475927, 0.933709, 0.353643, 0.226333, 0.270564, 1.265400, 0.438715,
	2.556685, 0.460857, 3.332732, 1.951951, 0.119152, 0.247955, 1.900739, 0.000000, 0.374834, 8.810038,
	0.180629, 5.230115, 1.565160, 0.316258, 0.171432, 0.331090, 0.461776, 0.286572, 1.745156,
	2.411739, 0.000000, 0.335419, 0.954557, 1.350599, 0.000000, 0.132142, 0.10385,
	0.921860, 0.170205, 0.619951, 1.031534, 0.000000, 0.000000, 2.565955,
	0.110506, 0.459901, 0.136655, 0.762354, 6.952629, 0.123606,
	2.427202, 0.782857, 0.000000, 0.000000, 0.485026,
	5.436674, 0.740819, 0.336289, 0.303836,
	0.000000, 0.417839, 1.561997,
	0.608070, 0.000000,
	0.279379
};

static double dayhoffFrequencies[NUM_AA] = {
	0.087127, 0.040904, 0.040432, 0.046872, 0.033474, 0.038255, 0.049530, 0.088612, 0.033619, 0.036886, 0.085357, 0.080481, 0.014753, 0.039772, 0.050680, 0.069577, 0.058542, 0.010494, 0.029916, 0.064718
};

/* BLOSUM62 model of amino acid evolution */
/* Henikoff, S., and J. G. Henikoff. 1992. PNAS USA 89:10915-10919.*/
static double blosumRelativeRates[NUM_AA_REL_RATES] = {
	7.3579038969751e-01, 4.8539105546575e-01, 5.4316182089867e-01, 1.4599953104700e+00, 1.1997057046020e+00, 1.1709490427999e+00, 1.9558835749595e+00, 7.1624144499779e-01, 6.0589900368677e-01, 8.0001653051838e-01, 1.2952012667833e+00, 1.2537582666635e+00, 4.9296467974759e-01, 1.1732759009239e+00, 4.3250926870566e+00, 1.7291780194850e+00, 4.6583936772479e-01, 7.1820669758623e-01, 2.1877745220045e+00, 
	1.2974467051337e+00, 5.0096440855513e-01, 2.2782657420895e-01, 3.0208336100636e+00, 1.3605741904203e+00, 4.1876330851753e-01, 1.4561411663360e+00, 2.3203644514174e-01, 6.2271166969249e-01, 5.4111151414889e+00, 9.8369298745695e-01, 3.7164469320875e-01, 4.4813366171831e-01, 1.1227831042096e+00, 9.1466595456337e-01, 4.2638231012175e-01, 7.2051744121611e-01, 4.3838834377202e-01, 
	3.1801000482161e+00, 3.9735894989702e-01, 1.8392161469920e+00, 1.2404885086396e+00, 1.3558723444845e+00, 2.4145014342081e+00, 2.8301732627800e-01, 2.1188815961519e-01, 1.5931370434574e+00, 6.4844127878707e-01, 3.5486124922252e-01, 4.9488704370192e-01, 2.9041016564560e+00, 1.8981736345332e+00, 1.9148204624678e-01, 5.3822251903674e-01, 3.1285879799342e-01, 
	2.4083661480204e-01, 1.1909457033960e+00, 3.7616252083685e+00, 7.9847324896839e-01, 7.7814266402188e-01, 4.1855573246161e-01, 2.1813157759360e-01, 1.0324479249521e+00, 2.2262189795786e-01, 2.8173069420651e-01, 7.3062827299842e-01, 1.5827541420653e+00, 9.3418750943056e-01, 1.4534504627853e-01, 2.6142220896504e-01, 2.5812928941763e-01, 
	3.2980150463028e-01, 1.4074889181440e-01, 4.1820319228376e-01, 3.5405810983129e-01, 7.7489402279418e-01, 8.3184264014158e-01, 2.8507880090648e-01, 7.6768882347954e-01, 4.4133747118660e-01, 3.5600849876863e-01, 1.1971884150942e+00, 1.1198313585160e+00, 5.2766441887169e-01, 4.7023773369610e-01, 1.1163524786062e+00, 
	5.5289191779282e+00, 6.0984630538281e-01, 2.4353411311401e+00, 2.3620245120365e-01, 5.8073709318144e-01, 3.9452776745146e+00, 2.4948960771127e+00, 1.4435695975031e-01, 8.5857057567418e-01, 1.9348709245965e+00, 1.2774802945956e+00, 7.5865380864172e-01, 9.5898974285014e-01, 5.3078579012486e-01, 
	4.2357999217628e-01, 1.6268910569817e+00, 1.8684804693170e-01, 3.7262517508685e-01, 2.8024271516787e+00, 5.5541539747043e-01, 2.9140908416530e-01, 9.2656393484598e-01, 1.7698932389373e+00, 1.0710972360073e+00, 4.0763564893830e-01, 5.9671930034577e-01, 5.2425384633796e-01, 
	5.3985912495418e-01, 1.8929629237636e-01, 2.1772115923623e-01, 7.5204244030271e-01, 4.5943617357855e-01, 3.6816646445253e-01, 5.0408659952683e-01, 1.5093262532236e+00, 6.4143601140497e-01, 5.0835892463812e-01, 3.0805573703500e-01, 2.5334079019018e-01, 
	2.5271844788492e-01, 3.4807220979697e-01, 1.0225070358890e+00, 9.8431152535870e-01, 7.1453370392764e-01, 5.2700733915060e-01, 1.1170297629105e+00, 5.8540709022472e-01, 3.0124860078016e-01, 4.2189539693890e+00, 2.0155597175031e-01, 
	3.8909637733035e+00, 4.0619358664202e-01, 3.3647977631042e+00, 1.5173593259539e+00, 3.8835540920564e-01, 3.5754441245967e-01, 1.1790911972601e+00, 3.4198578754023e-01, 6.7461709322842e-01, 8.3118394054582e+00, 
	4.4557027426059e-01, 6.0305593795716e+00, 2.0648397032375e+00, 3.7455568747097e-01, 3.5296918452729e-01, 9.1525985769421e-01, 6.9147463459998e-01, 8.1124585632307e-01, 2.2314056889131e+00, 
	1.0730611843319e+00, 2.6692475051102e-01, 1.0473834507215e+00, 1.7521659178195e+00, 1.3038752007987e+00, 3.3224304063396e-01, 7.1799348690032e-01, 4.9813847530407e-01, 
	1.7738551688305e+00, 4.5412362510273e-01, 9.1872341574605e-01, 1.4885480537218e+00, 8.8810109815193e-01, 9.5168216224591e-01, 2.5758507553153e+00, 
	2.3359790962888e-01, 5.4002764482413e-01, 4.8820611879305e-01, 2.0743248934965e+00, 6.7472604308008e+00, 8.3811961017754e-01, 
	1.1691295777157e+00, 1.0054516831488e+00, 2.5221483002727e-01, 3.6940531935451e-01, 4.9690841067567e-01, 
	5.1515562922704e+00, 3.8792562209837e-01, 7.9675152076106e-01, 5.6192545744165e-01, 
	5.1312812689059e-01, 8.0101024319939e-01, 2.2530740511763e+00, 
	4.0544190065580e+00, 2.6650873142646e-01, 
	1.0000000000000e+00
};

static double blosumFrequencies[NUM_AA] = {
	0.074, 0.052, 0.045, 0.054, 0.025, 0.034, 0.054, 0.074, 0.026, 0.068, 0.099, 0.058, 0.025, 0.047, 0.039, 0.057, 0.051, 0.013, 0.032, 0.073
};

/* mtRev model - complete sequence data of mtDNA from 24 vertebrate species */
/* Adachi, J., and Hasegawa, M. 1996. J. Mol. Evol. 42:459-468. */
static double mtrevRelativeRates[NUM_AA_REL_RATES] = {
	1.2199217606346e+01, 1.4182139942122e+01, 9.2985091873208e+00, 3.1542792981957e+01, 1.0025852846688e+00, 5.1418866803338e+00, 6.3531246495131e+01, 7.3137132861715e+00, 5.0782382656186e+01, 1.3399741808481e+01, 4.4021672780560e+00, 7.4673480520104e+01, 3.3513021631978e+00, 2.8582502221773e+01, 2.0413623195312e+02, 2.5301305153906e+02, 1.0000000000000e+00, 3.4084158197615e+00, 1.0266468401249e+02, 
	6.9661274444534e+00, 1.0000000000000e+00, 5.4384584796568e+01, 1.1631134513343e+02, 1.0000000000000e+00, 1.2122831341194e+01, 8.6961067087353e+01, 1.0000000000000e+00, 8.1976829394538e+00, 7.4423215395318e+01, 1.0000000000000e+00, 2.4659158338099e+00, 1.2439947713615e+01, 3.1791814866372e+00, 1.0935327216119e+00, 1.1550775790126e+01, 1.0000000000000e+00, 4.0211417480338e+00, 
	4.1809325468160e+02, 3.1020979842967e+01, 9.1349622725361e+01, 3.3185663516310e+01, 2.8052324651124e+01, 2.6112087577885e+02, 1.4261453863336e+01, 7.9775653461977e+00, 3.2036829276162e+02, 3.4424354918739e+01, 7.9996445145608e+00, 3.8586541461044e+01, 2.6020426225852e+02, 1.2550758780474e+02, 5.6207759736659e+00, 1.0071406219571e+02, 1.0000000000000e+00, 
	1.0000000000000e+00, 2.9097352675564e+01, 3.0713149855302e+02, 2.9877072751897e+01, 5.9995408885817e+01, 2.2827096245105e+00, 1.0000000000000e+00, 1.2183938185384e+00, 1.0000000000000e+00, 2.6221929413096e+00, 7.0708004204733e+00, 3.6327934317139e+01, 1.4743408713748e+01, 1.0453246057102e+01, 1.1165627147496e+01, 1.0000000000000e+00, 
	3.9599394038972e+01, 1.0000000000000e+00, 1.6163581056674e+01, 7.4467985406234e+01, 3.3018175376623e+01, 1.3500725995091e+01, 1.0000000000000e+00, 3.2504095376923e+00, 3.7264767083096e+01, 1.6454136037822e+01, 1.4581783243113e+02, 9.4720031458442e+01, 1.7684087896962e+01, 1.3409157685926e+02, 1.0000000000000e+00, 
	1.6503249008836e+02, 3.5530760735494e+00, 3.0652523140859e+02, 4.3905393139325e+00, 2.0895470525345e+01, 2.4504076430724e+02, 2.4931300477797e+01, 1.0059428264289e+01, 7.2256314165467e+01, 2.8480937892158e+01, 4.9962974409828e+01, 1.0000000000000e+00, 2.0430790980529e+01, 9.9986289000676e+00, 
	1.4884496769963e+01, 2.5853576435567e+01, 1.7418201388328e+00, 1.0000000000000e+00, 1.6519126809071e+02, 1.0000000000000e+00, 1.4067850525292e+00, 6.7547121641947e+00, 2.8794794140840e+01, 7.8001372062558e+00, 1.0000000000000e+00, 6.9067239183061e+00, 1.1127702362585e+01, 
	1.0000000000000e+00, 3.1466649021550e+00, 1.2699794194865e+00, 1.1962111069278e+01, 1.0000000000000e+00, 1.0000000000000e+00, 1.0000000000000e+00, 6.6277950574411e+01, 5.8800079133028e+00, 5.7494182626674e+00, 1.6887657206208e+00, 1.3320553471351e+00, 
	6.4536986087271e+00, 6.0472584534958e+00, 6.7197196398961e+01, 6.2977633277779e+00, 2.5347805183364e+01, 3.2089868698728e+01, 4.0766987134407e+01, 2.3570850628539e+01, 3.7286635325194e+00, 3.5270764890474e+02, 1.0000000000000e+00, 
	1.7320653206333e+02, 1.0298655619743e+01, 2.7262244199514e+02, 4.4561065036310e+01, 1.0856482766156e+01, 2.5107659603898e+01, 1.9391167162525e+02, 1.0000000000000e+00, 1.3161329199391e+01, 6.4365086389428e+02, 
	7.8314019154706e+00, 2.8290920517725e+02, 1.1371735519833e+02, 2.1105885757279e+01, 3.8741359395934e+01, 6.6524559321657e+01, 1.7071378554833e+01, 2.3234516108847e+01, 4.8247261078055e+01, 
	4.8092094826036e+01, 3.3887559483420e+00, 2.6368577564199e+01, 5.5679895711418e+01, 7.1750284708933e+01, 1.2631893872825e+01, 2.6932728996777e+01, 1.0000000000000e+00, 
	4.7798798034572e+01, 9.9165053447429e+00, 5.8505442466161e+01, 2.7798190504760e+02, 1.1427000119701e+01, 2.1029990530586e+01, 2.0397078683768e+02, 
	9.1089574817139e+00, 3.3835737720574e+01, 1.7815549567056e+01, 4.1272404968214e+00, 2.4504156395152e+02, 3.3435675442163e+00, 
	8.9421193040709e+01, 6.7485067008375e+01, 2.2161693733113e+00, 8.5338209390745e+00, 4.3342126659660e+00, 
	3.1432036618746e+02, 2.0305343047059e+01, 3.4167877957799e+01, 1.0000000000000e+00, 
	5.2559565123081e+00, 2.0382362288681e+01, 1.0765527137500e+02, 
	1.3814733274637e+01, 2.8259139240676e+00, 
	1.0000000000000e+00
};

static double mtrevFrequencies[NUM_AA] = {
	0.072, 0.019, 0.039, 0.019, 0.006, 0.025, 0.024, 0.056, 0.028, 0.088, 0.168, 0.023, 0.054, 0.061, 0.054, 0.072, 0.086, 0.029, 0.033, 0.043
};

/* CPREV 45 model of amino acid evolution */
/* Adachi, J., P.J. Waddell, W. Martin, and M. Hasegawa. 2000. JME 50:348-358 */
static double cprevRelativeRates[NUM_AA_REL_RATES] = {
	-105, -227, -175, -669, -157, -499, -665, -66, -145, -197, -236, -185, -68, -490, -2440, -1340, -14, -56, -968, 
	-357, -43, -823, -1745, -152, -243, -715, -136, -203, -4482, -125, -53, -87, -385, -314, -230, -323, -92, 
	-4435, -538, -768, -1055, -653, -1405, -168, -113, -2430, -61, -97, -173, -2085, -1393, -40, -754, -83, 
	-10, -400, -3691, -431, -331, -10, -10, -412, -47, -22, -170, -590, -266, -18, -281, -75, 
	-10, -10, -303, -441, -280, -396, -48, -159, -726, -285, -2331, -576, -435, -1466, -592, 
	-3122, -133, -1269, -92, -286, -3313, -202, -10, -323, -396, -241, -53, -391, -54, 
	-379, -162, -148, -82, -2629, -113, -145, -185, -568, -369, -63, -142, -200, 
	-19, -40, -20, -263, -21, -25, -28, -691, -92, -82, -10, -91, 
	-29, -66, -305, -10, -127, -152, -303, -32, -69, -1971, -25, 
	-1745, -345, -1772, -454, -117, -216, -1040, -42, -89, -4797, 
	-218, -1351, -1268, -219, -516, -156, -159, -189, -865, 
	-193, -72, -302, -868, -918, -10, -247, -249, 
	-327, -100, -93, -645, -86, -215, -475, 
	-43, -487, -148, -468, -2370, -317, 
	-1202, -260, -49, -97, -122, 
	-2151, -73, -522, -167, 
	-29, -71, -760, 
	-346, -10, 
	-119
};

static double cprevFrequencies[NUM_AA] = {
	0.076, 0.062, 0.041, 0.037, 0.009, 0.038, 0.049, 0.084, 0.025, 0.081, 0.101, 0.050, 0.022, 0.051, 0.043, 0.062, 0.054, 0.018, 0.031, 0.066
};

/*
MtArt model inserted in following

7 Oct 2011 -- Lars Jermiin (lars.jermiin@csiro.au)
*/

/* MtArt: A New Model of Amino Acid Replacement for Arthropoda */
/* Abascal, F., Posada, D., and Zardoya, R. 2007. Mol. Biol. Evol. 24:1-5. */
static double mtartRelativeRates[NUM_AA_REL_RATES] = {
	0.19, 0.19, 0.59, 253.51, 0.19, 0.19, 199.84, 0.19, 25.69, 3.70, 0.19, 120.64, 13.10, 49.33, 672.97, 243.93, 0.19, 1.18, 339.91,
	0.19, 4.28, 35.51, 153.97, 0.19, 0.19, 41.30, 1.81, 1.77, 208.60, 5.18, 4.73, 0.19, 2.66, 0.19, 0.19, 3.88, 0.19,
	500.16, 98.18, 261.79, 183.04, 120.53, 179.54, 21.33, 12.65, 467.34, 78.81, 19.72, 16.54, 398.43, 165.89, 7.73, 251.17, 22.60,
	10.61, 0.19, 861.83, 12.46, 0.19, 6.60, 1.17, 1.68, 0.19, 0.19, 0.19, 44.35, 0.19, 0.19, 0.19, 0.19,
	0.19, 0.19, 80.53, 12.37, 63.00, 78.71, 0.19, 312.30, 184.06, 0.19, 664.16, 182.80, 21.61, 71.99, 350.40,
	261.55, 2.64, 313.50, 10.54, 16.28, 349.28, 67.33, 0.19, 39.30, 52.36, 43.68, 6.68, 86.67, 0.19,
	43.86, 15.25, 6.81, 1.72, 106.31, 0.19, 0.19, 7.88, 31.50, 43.40, 10.99, 7.74, 13.63,
	0.19, 2.74, 1.36, 0.19, 55.71, 0.83, 0.19, 226.03, 0.19, 1.88, 8.65, 2.56,
	0.19, 5.54, 0.19, 0.19, 13.78, 0.79, 10.62, 18.59, 0.19, 191.43, 0.19,
	514.54, 3.46, 514.78, 117.91, 0.19, 7.15, 203.74, 0.19, 12.33, 1854.52,
	3.78, 885.50, 262.57, 12.17, 8.17, 47.75, 21.13, 19.76, 84.72,
	105.61, 10.73, 16.82, 144.22, 69.54, 15.97, 117.09, 26.06,
	321.61, 5.26, 111.74, 288.57, 70.68, 70.92, 281.29,
	14.62, 36.05, 13.54, 53.67, 791.56, 51.90,
	86.50, 46.83, 0.19, 18.39, 31.70,
	660.39, 2.38, 30.45, 60.59,
	0.19, 45.98, 544.14,
	37.72, 0.19,
	1.59
};

static double mtartFrequencies[NUM_AA] = {
	0.054116, 0.018227, 0.039903, 0.020160, 0.009709, 0.018781, 0.024289, 0.068183, 0.024518, 0.092639, 0.148658, 0.021718, 0.061453, 0.088668, 0.041826, 0.091030, 0.049194, 0.029786, 0.039443, 0.057701
};

/*************************************/
void SetAAModel(int theAAModel)
{	
	aaModel = theAAModel;
	
/*
"AA_MTART" inserted in position 6 of the following switch
 
7 Oct 2011 -- Lars Jermiin (lars.jermiin@csiro.au)
*/
	switch (aaModel) {
		case AA_JTT: SetRelativeRates(jttRelativeRates); break;
		case AA_WAG: SetRelativeRates(wagRelativeRates); break; 
	        case AA_LG: initilizeLG(); SetRelativeRates(LGRelativeRates); break;
		case AA_DAYHOFF78: SetRelativeRates(dayhoffRelativeRates); break;
		case AA_BLOSUM62: SetRelativeRates(blosumRelativeRates); break;
		case AA_MTREV24: SetRelativeRates(mtrevRelativeRates); break;
		case AA_CPREV45: SetRelativeRates(cprevRelativeRates); break;
		case AA_MTART: SetRelativeRates(mtartRelativeRates); break;
		case AA_GENERAL: 
		  /* relative rates set by user */
		break;
	}

/*
"AA_MTART" inserted in position 6 of the following switch
 
7 Oct 2011 -- Lars Jermiin (lars.jermiin@csiro.au)
*/
	if (!aaFreqSet) {
		switch (aaModel) {
			case AA_JTT: SetFrequencies(jttFrequencies); break;
			case AA_WAG: SetFrequencies(wagFrequencies); break;
			case AA_LG: SetFrequencies(LGFrequencies); break;
		        case AA_DAYHOFF78: SetFrequencies(dayhoffFrequencies); break;
			case AA_BLOSUM62: SetFrequencies(blosumFrequencies); break;
			case AA_MTREV24: SetFrequencies(mtrevFrequencies); break;
			case AA_CPREV45: SetFrequencies(cprevFrequencies); break;
			case AA_MTART: SetFrequencies(mtartFrequencies); break;
			case AA_GENERAL:
			  /* frequencies set by user */
			break;
		}
		
		if (user_defined_aafreq==1) 
		  getAAfreq(aaFreqFileName);  /* over-write the model freq */
	}
	
	else {
	  /* User set or equal frequencies */
		CheckAAFrequencies();
	}
	
	SetupAAMatrix();
}


int getSiteAAfreq(char *filename, int site)
{
  FILE *freqinfile;
  int i=0, count=-1;
  char buf[500], *pch;

  if ((freqinfile = fopen (filename, "r"))==NULL) {
    printf("\n\nError: target site freq file: %s is not found\n\n", filename);
    exit(0);
    return 0;
  }

  while (fgets(buf,500,freqinfile) !=NULL) {
    count++;
    if (count == site) {
      pch = strtok (buf, " ");
      while (pch != NULL) {
	aaFreq[i++] = atof(pch);
	pch = strtok (NULL, " ");
      }
    }
    
  }
  fclose(freqinfile);
  return 1;
}

int getAAfreq(char *filename)
{
  FILE *freqinfile;
  int i=0;
  char buf[500], *pch;

  if ((freqinfile = fopen (filename, "r"))==NULL) {
    printf("\n\nError: A user-defined aa freq file: %s is not found\n\n", filename);
    exit(0);
    return 0;
  }

  while (fgets(buf,500,freqinfile) !=NULL) {
      pch = strtok (buf, " ");
      while (pch != NULL) {
	aaFreq[i++] = atof(pch);
	pch = strtok (NULL, " ");
      }
  }
    
  fclose(freqinfile);
  return 1;
}


void getaaFreq(char *filename, int theAAModel) {
  int i;
  aaModel = theAAModel;

  if (user_defined_aafreq==0) {
    switch (aaModel) {
    case AA_JTT: SetFrequencies(jttFrequencies); break;
    case AA_WAG: SetFrequencies(wagFrequencies); break;
    case AA_LG: SetFrequencies(LGFrequencies); break;
    case AA_DAYHOFF78: SetFrequencies(dayhoffFrequencies); break;
    case AA_BLOSUM62: SetFrequencies(blosumFrequencies); break;
    case AA_MTREV24: SetFrequencies(mtrevFrequencies); break;
    case AA_CPREV45: SetFrequencies(cprevFrequencies); break;
    case AA_MTART: SetFrequencies(mtartFrequencies); break;
    case AA_GENERAL:
      /* frequencies set by user */
      break;
    }
  }

  else getAAfreq(filename);

}


void SetAAModel1(int theAAModel, int partition)
{	
	
  int i;
  aaModel = theAAModel;
	
/*
"AA_MTART" inserted in position 6 of the following switch
 
7 Oct 2011 -- Lars Jermiin (lars.jermiin@csiro.au)
*/
	switch (aaModel) {
		case AA_JTT: SetRelativeRates(jttRelativeRates); break;
		case AA_WAG: SetRelativeRates(wagRelativeRates); break;
	        case AA_LG: initilizeLG(); SetRelativeRates(LGRelativeRates); break;
		case AA_DAYHOFF78: SetRelativeRates(dayhoffRelativeRates); break;
		case AA_BLOSUM62: SetRelativeRates(blosumRelativeRates); break;
		case AA_MTREV24: SetRelativeRates(mtrevRelativeRates); break;
		case AA_CPREV45: SetRelativeRates(cprevRelativeRates); break;
		case AA_MTART: SetRelativeRates(mtartRelativeRates); break;
		case AA_GENERAL: 
		  /* relative rates set by user  */
		break;
	}

/*
"AA_MTART" inserted in position 6 of the following switch
 
7 Oct 2011 -- Lars Jermiin (lars.jermiin@csiro.au)
*/
		
	if (!aaFreqSet) {
		switch (aaModel) {
			case AA_JTT: SetFrequencies(jttFrequencies); break;
			case AA_WAG: SetFrequencies(wagFrequencies); break;
			case AA_LG: SetFrequencies(LGFrequencies); break;
		        case AA_DAYHOFF78: SetFrequencies(dayhoffFrequencies); break;
			case AA_BLOSUM62: SetFrequencies(blosumFrequencies); break;
			case AA_MTREV24: SetFrequencies(mtrevFrequencies); break;
			case AA_CPREV45: SetFrequencies(cprevFrequencies); break;
			case AA_MTART: SetFrequencies(mtartFrequencies); break;
			case AA_GENERAL:
			  /* frequencies set by user */
			break;
		}
		if (user_defined_aafreq==1) 
		  getAAfreq(aaFreqFileName);  /* over-write the model freq */
	} 
	else {
	  /* each site has its freq vector */
	  if (SSF == 1) {
	    if (!getSiteAAfreq(site_aafreqFileName, partition))
	    printf("\nwrong partitions: partition no = %d\n", partition);
	  }
	  else if (C20 == 1) {
	    if (partition > 0) 
	      for (i=0; i<20; i++)
		aaFreq[i] = C20Frequencies[partition-1][i];
	    else if (partition == 0)  /* for partition 0 we use the protein model frequencies or a user-defined aa freqs*/
	      getaaFreq(aaFreqFileName, aaModel);
	      
	    else printf("\nwrong partitions: partition no = %d\n", partition);
	  }
	  else if (C60 == 1) {
	    if (partition > 0) 
	      for (i=0; i<60; i++)
		aaFreq[i] = C60Frequencies[partition-1][i];
	    else if (partition == 0)  /* for partition 0 we use the protein model frequencies or a user-defined aa freqs*/
	      getaaFreq(aaFreqFileName, aaModel);

	    else printf("\nwrong partitions: partition no = %d\n", partition);
	  }
	  
	  CheckAAFrequencies();
	}
	
	SetupAAMatrix();
}
		

void SetRelativeRates(double *inRelativeRate) 
{
	int i;
	for (i=0; i<NUM_AA_REL_RATES; i++) {
		aaRelativeRate[i] = inRelativeRate[i];
	}
}

void SetFrequencies(double *inFrequencies)
{
	int i;
	for (i=0; i<NUM_AA; i++) {
		aaFreq[i] = inFrequencies[i];
	}
}

/* decompose Q matrix: Qij*AAfreq */
void SetupAAMatrix()
{
	int i,j,k;
	double mr;
	double sum;
	double U[SQNUM_AA], V[SQNUM_AA], T1[SQNUM_AA], T2[SQNUM_AA];

	k=0;
	for (i=0; i<NUM_AA-1; i++) {
		for (j=i+1; j<NUM_AA; j++) {
			Qij[i*NUM_AA+j] = Qij[j*NUM_AA+i] = aaRelativeRate[k++];
		}
	}
	
	for (i=0; i<NUM_AA; i++) {
		for (j=0; j<NUM_AA; j++) { 
			Qij[i*NUM_AA+j] *= aaFreq[j];
		}
	}

	mr=0;		
	for (i=0; i<NUM_AA; i++) {
		sum = 0;
		Qij[i*NUM_AA+i]=0; 
		for (j=0; j<NUM_AA; j++) { 
			sum += Qij[i*NUM_AA+j];
		}
		Qij[i*NUM_AA+i] = -sum; 
		mr += aaFreq[i] * sum;
	}
	
	abyx(1.0/mr, Qij, SQNUM_AA);

	if ((k=eigen(1, Qij, NUM_AA, Root, T1, U, V, T2))!=0) {
		fprintf(stderr, "\ncomplex roots in SetupAAMatrix");
		exit(0);
	}
	xtoy (U, V, SQNUM_AA);
	matinv (V, NUM_AA, NUM_AA, T1);
	for (i=0; i<NUM_AA; i++) {
   		for (j=0; j<NUM_AA; j++) {
   			for (k=0; k<NUM_AA; k++) {
   				Cijk[i*SQNUM_AA+j*NUM_AA+k] = U[i*NUM_AA+k]*V[k*NUM_AA+j];
   			}
   		}
   	}
}

/**
 * Ensures that frequencies are not smaller than MINFREQ and
 * that two frequencies differ by at least 2*MINFDIFF.
 * This avoids potential problems later when eigenvalues
 * are computed.
 */
void CheckAAFrequencies()
{
	int i, j;
	double diff;
	
	/* required frequency difference*/
	double MINFDIFF = 1.0E-10;

	/* lower limit on frequency*/
	double MINFREQ = 1.0E-10;

	int maxi = 0;
	double sum = 0.0;
	double maxfreq = 0.0;
	for (i = 0; i < NUM_AA; i++) {
		if (aaFreq[i] < MINFREQ) aaFreq[i] = MINFREQ;
		if (aaFreq[i] > maxfreq) {
			maxfreq = aaFreq[i];
			maxi = i;
		}
		sum += aaFreq[i];
	}
	
	diff = 1.0 - sum;
	aaFreq[maxi] += diff;

	for (i = 0; i < NUM_AA - 1; i++) {
		for (j = i+1; j < NUM_AA; j++) {
			if (aaFreq[i] == aaFreq[j]) {
				aaFreq[i] += MINFDIFF;
				aaFreq[j] -= MINFDIFF;
			}
		}
	}
}

void SetAAMatrix(double *matrix, double len)
{	
	int i,j,k;
	double expt[NUM_AA];
	double *P;
	
/* P(t)ij = SUM Cijk * exp{Root*t}
*/
	P=matrix;
	if (len<1e-6) { 
		for (i=0; i<NUM_AA; i++) {
			for (j=0; j<NUM_AA; j++) {
				if (i==j)
					*P=1.0;
				else 	
					*P=0.0;
				P++;
			}
		}
		return; 
	}
	
	for (k=1; k<NUM_AA; k++) {
		expt[k]=exp(len*Root[k]);
	}
	for (i=0; i<NUM_AA; i++) {
		for (j=0; j<NUM_AA; j++) {
			(*P)=Cijk[i*SQNUM_AA+j*NUM_AA];
			for (k=1; k<NUM_AA; k++) {
				(*P)+=Cijk[i*SQNUM_AA+j*NUM_AA+k]*expt[k];
			}
			P++;
		}
	}
	
	P=matrix;
	for (i=0; i<NUM_AA; i++) {
		P++;
		for (j=1; j<NUM_AA; j++) {
			*P += *(P - 1);
			P++;
		}
	}
}

void SetAAVector(double *vector, short state, double len)
{	
	int i,j,k;
	double expt[NUM_AA];
	double *P;

	P=vector;
	if (len<1e-6) { 
		for (i=0; i<NUM_AA; i++) {
			if (i==state)
				*P=1.0;
			else 	
				*P=0.0;
			P++;
		}
		return; 
	}
	for (k=1; k<NUM_AA; k++) {
		expt[k]=exp(len*Root[k]);
	}
	
	for (j=0; j<NUM_AA; j++) {
		(*P)=Cijk[state*SQNUM_AA+j*NUM_AA];
		for (k=1; k<NUM_AA; k++) {
			(*P)+=Cijk[state*SQNUM_AA+j*NUM_AA+k]*expt[k];
		}
		P++;
	}
	
	P=vector;
	P++;
	for (j=1; j<NUM_AA; j++) {
		*P += *(P - 1);
		P++;
	}
}


